@@ -2,7 +2,7 @@
 ** ColorSpecialHandler.h                                                **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2013 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,18 +18,17 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#ifndef COLORSPECIALHANDLER_H
-#define COLORSPECIALHANDLER_H
+#ifndef DVISVGM_COLORSPECIALHANDLER_H
+#define DVISVGM_COLORSPECIALHANDLER_H
 
 #include <stack>
 #include <vector>
+#include "Color.h"
 #include "SpecialHandler.h"
 
 
 class ColorSpecialHandler : public SpecialHandler
 {
-	typedef std::vector<float> RGB;
-
    public:
 		bool process (const char *prefix, std::istream &is, SpecialActions *actions);
 		const char* name () const   {return "color";}
@@ -37,7 +36,7 @@ class ColorSpecialHandler : public SpecialHandler
 		const char** prefixes () const;
 
 	private:
-		std::stack<RGB> _colorStack;
+		std::stack<Color> _colorStack;
 };
 
 #endif
