@@ -2,7 +2,7 @@
 ** DVIReader.cpp                                                        **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2013 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,11 +18,12 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
+#include <config.h>
 #include <cstdarg>
 #include <fstream>
 #include <iostream>
 #include <sstream>
-#include "types.h"
+#include "Color.h"
 #include "DVIActions.h"
 #include "DVIReader.h"
 #include "Font.h"
@@ -31,17 +32,19 @@
 #include "SignalHandler.h"
 #include "VectorStream.h"
 #include "macros.h"
+#include "types.h"
+
 
 using namespace std;
 
 bool DVIReader::COMPUTE_PAGE_LENGTH = false;
 
 
-DVIReader::DVIReader (istream &is, DVIActions *a) : StreamReader(is), _actions(a)
+DVIReader::DVIReader (istream &is, DVIActions *a) : StreamReader(is), _dviFormat(DVI_NONE), _actions(a)
 {
 	_inPage = false;
 	_pageHeight = _pageWidth = 0;
-	_scaleFactor = 0.0;
+	_dvi2bp = 0.0;
 	_tx = _ty = 0;    // no cursor translation
 	_prevYPos = numeric_limits<double>::min();
 	_inPostamble = false;
@@ -50,6 +53,8 @@ DVIReader::DVIReader (istream &is, DVIActions *a) : StreamReader(is), _actions(a
 	_currPageNum = 0;
 	_pageLength = 0;
 	_pagePos = 0;
+	_prevBop = 0;
+	_mag = 1;
 }
 
 
@@ -63,7 +68,7 @@ DVIActions* DVIReader::replaceActions (DVIActions *a) {
 /** Evaluates the next DVI command, and computes the corresponding handler.
  *  @param[in] compute_size if true, the size of variable-length parameters is computed
  *  @param[out] handler handler for current DVI command
- *  @param[out] number of parameter bytes
+ *  @param[out] length number of parameter bytes
  *  @param[out] param the handler must be called with this parameter
  *  @return opcode of current DVI command */
 int DVIReader::evalCommand (bool compute_size, CommandHandler &handler, int &length, int &param) {
@@ -96,7 +101,8 @@ int DVIReader::evalCommand (bool compute_size, CommandHandler &handler, int &len
 		{&DVIReader::cmdFontNum, 1}, {&DVIReader::cmdFontNum, 2}, {&DVIReader::cmdFontNum, 3}, {&DVIReader::cmdFontNum, 4}, // 235-238
 		{&DVIReader::cmdXXX, 1},     {&DVIReader::cmdXXX, 2},     {&DVIReader::cmdXXX, 3},     {&DVIReader::cmdXXX, 4},     // 239-242
 		{&DVIReader::cmdFontDef, 1}, {&DVIReader::cmdFontDef, 2}, {&DVIReader::cmdFontDef, 3}, {&DVIReader::cmdFontDef, 4}, // 243-246
-		{&DVIReader::cmdPre, 0},     {&DVIReader::cmdPost, 0},    {&DVIReader::cmdPostPost, 0}                              // 247-249
+		{&DVIReader::cmdPre, 0},     {&DVIReader::cmdPost, 0},    {&DVIReader::cmdPostPost, 0},                             // 247-249
+		{&DVIReader::cmdXPic, 0},    {&DVIReader::cmdXFontDef, 0},{&DVIReader::cmdXGlyphA, 0}, {&DVIReader::cmdXGlyphS, 0}  // 251-254
 	};
 
 	const int opcode = in().get();
@@ -114,6 +120,20 @@ int DVIReader::evalCommand (bool compute_size, CommandHandler &handler, int &len
 		length = 0;
 		param = opcode-171;
 	}
+	else if (opcode >= 251 && opcode <= 254 && _dviFormat == DVI_XDV) {  // XDV command?
+		static const CommandHandler handlers[] = {
+			&DVIReader::cmdXPic,
+			&DVIReader::cmdXFontDef,
+			&DVIReader::cmdXGlyphA,
+			&DVIReader::cmdXGlyphS
+		};
+		handler = handlers[opcode-251];
+		param = 0;
+	}
+	else if (opcode == 255 && _dviFormat == DVI_PTEX) {  // direction command set by pTeX?
+		handler = &DVIReader::cmdDir;
+		length = 1;
+	}
 	else if (opcode >= 250) {
 		ostringstream oss;
 		oss << "undefined DVI command (opcode " << opcode << ')';
@@ -161,9 +181,9 @@ int DVIReader::executeCommand () {
 	streampos pos = in().tellg();
 	int opcode = evalCommand(false, handler, len, param);
 	(this->*handler)(param);
-	if (_currPos.v+_ty != _prevYPos) {
+	if (_dviState.v+_ty != _prevYPos) {
 		_tx = _ty = 0;
-		_prevYPos = _currPos.v;
+		_prevYPos = _dviState.v;
 	}
 	if (COMPUTE_PAGE_LENGTH && _inPage && _actions) {
 		// ensure progress() is called at 0%
@@ -195,28 +215,6 @@ void DVIReader::executeAll () {
 }
 
 
-#if 0
-/** Executes all DVI commands from the preamble to postpost. */
-bool DVIReader::executeDocument () {
-	in().clear();   // reset all status bits
-	if (!in())
-		return false;
-	in().seekg(0);  // move file pointer to first byte of the input stream
-	while (!in().eof() && executeCommand() != 249); // stop reading after postpost (249)
-	return true;
-}
-
-bool DVIReader::executeAllPages () {
-	in().clear();   // reset all status bits
-	if (!in())
-		return false;
-	in().seekg(0);  // move file pointer to first byte of the input stream
-	while (!in().eof() && executeCommand() != 248); // stop reading when postamble (248) is reached
-	return true;
-}
-#endif
-
-
 /** Reads and executes the commands of a single page.
  *  This methods stops reading after the page's eop command has been executed.
  *  @param[in] n number of page to be executed
@@ -236,10 +234,10 @@ bool DVIReader::executePage (unsigned n) {
 	if (n < 1 || n > _totalPages)
 		return false;
 	in().seekg(_prevBop, ios_base::beg); // now on last bop
-	_inPostamble = false;               // we jumped out of the postamble
+	_inPostamble = false;                // we jumped out of the postamble
 	unsigned pageCount = _totalPages;
 	for (; pageCount > n && _prevBop > 0; pageCount--) {
-		in().seekg(41, ios_base::cur);   // skip bop and 10*4 \count bytes => now on pointer to prev bop
+		in().seekg(41, ios_base::cur);    // skip bop and 10*4 \count bytes => now on pointer to prev bop
 		_prevBop = readSigned(4);
 		in().seekg(_prevBop, ios_base::beg);
 	}
@@ -266,10 +264,10 @@ bool DVIReader::executePages (unsigned first, unsigned last) {
 	first = max(1u, min(first, _totalPages));
 	last = max(1u, min(last, _totalPages));
 	in().seekg(_prevBop, ios_base::beg); // now on last bop
-	_inPostamble = false;               // we jumped out of the postamble
+	_inPostamble = false;                // we jumped out of the postamble
 	unsigned count = _totalPages;
 	for (; count > first && _prevBop > 0; count--) {
-		in().seekg(41, ios_base::cur);   // skip bop and 10*4 \count bytes => now on pointer to prev bop
+		in().seekg(41, ios_base::cur);    // skip bop and 10*4 \count bytes => now on pointer to prev bop
 		_prevBop = readSigned(4);
 		in().seekg(_prevBop, ios_base::beg);
 	}
@@ -308,17 +306,17 @@ void DVIReader::executePostamble () {
 }
 
 
-/** Returns the current x coordinate in TeX point units.
+/** Returns the current x coordinate in PS point units.
  *  This is the horizontal position where the next output would be placed. */
 double DVIReader::getXPos () const {
-	return _currPos.h+_tx;
+	return _dviState.h+_tx;
 }
 
 
-/** Returns the current y coordinate in TeX point units.
+/** Returns the current y coordinate in PS point units.
  *  This is the vertical position where the next output would be placed. */
 double DVIReader::getYPos () const {
-	return _currPos.v+_ty;
+	return _dviState.v+_ty;
 }
 
 
@@ -331,24 +329,36 @@ double DVIReader::getPageWidth () const {
 	return _pageWidth;
 }
 
+
+void DVIReader::verifyDVIFormat (int id) const {
+	switch (id) {
+		case DVI_STANDARD:
+		case DVI_PTEX:
+		case DVI_XDV:
+			break;
+		default:
+			ostringstream oss;
+			oss << "DVI format version " << id << " not supported";
+			throw DVIException(oss.str());
+	}
+}
+
 /////////////////////////////////////
 
 /** Reads and executes DVI preamble command. */
 void DVIReader::cmdPre (int) {
-	UInt32 i   = readUnsigned(1);  // identification number (should be 2)
+	_dviFormat = max(_dviFormat, (DVIFormat)readUnsigned(1)); // identification number
+	verifyDVIFormat(_dviFormat);
 	UInt32 num = readUnsigned(4);  // numerator units of measurement
 	UInt32 den = readUnsigned(4);  // denominator units of measurement
+	if (den == 0)
+		throw DVIException("denominator of measurement unit is zero");
 	_mag = readUnsigned(4);        // magnification
 	UInt32 k   = readUnsigned(1);  // length of following comment
 	string cmt = readString(k);    // comment
-	if (i != 2) {
-		ostringstream oss;
-		oss << "DVI format version " << i << " not supported";
-		throw DVIException(oss.str());
-	}
 	// 1 dviunit * num/den == multiples of 0.0000001m
-	// 1 dviunit * _scaleFactor: length of 1 dviunit in TeX points * _mag/1000
-	_scaleFactor = num/25400000.0*7227.0/den*_mag/1000.0;
+	// 1 dviunit * _dvibp: length of 1 dviunit in PS points * _mag/1000
+	_dvi2bp = num/254000.0*72.0/den*_mag/1000.0;
 	if (_actions)
 		_actions->preamble(cmt);
 }
@@ -359,16 +369,18 @@ void DVIReader::cmdPost (int) {
 	_prevBop   = readUnsigned(4);  // pointer to previous bop
 	UInt32 num = readUnsigned(4);
 	UInt32 den = readUnsigned(4);
+	if (den == 0)
+		throw DVIException("denominator of measurement unit is zero");
 	_mag = readUnsigned(4);
 	_pageHeight = readUnsigned(4); // height of tallest page in dvi units
 	_pageWidth  = readUnsigned(4); // width of widest page in dvi units
 	readUnsigned(2);               // max. stack depth
 	_totalPages = readUnsigned(2); // total number of pages
 	// 1 dviunit * num/den == multiples of 0.0000001m
-	// 1 dviunit * _scaleFactor: length of 1 dviunit in TeX points * _mag/1000
-	_scaleFactor = num/25400000.0*7227.0/den*_mag/1000.0;
-	_pageHeight *= _scaleFactor;     // to pt units
-	_pageWidth *= _scaleFactor;
+	// 1 dviunit * _dvi2bp: length of 1 dviunit in PS points * _mag/1000
+	_dvi2bp = num/254000.0*72.0/den*_mag/1000.0;
+	_pageHeight *= _dvi2bp;   // to pt units
+	_pageWidth *= _dvi2bp;
 	_inPostamble = true;
 	if (_actions)
 		_actions->postamble();
@@ -378,12 +390,10 @@ void DVIReader::cmdPost (int) {
 /** Reads and executes DVI postpost command. */
 void DVIReader::cmdPostPost (int) {
 	_inPostamble = false;
-	readUnsigned(4);               // pointer to begin of postamble
-	UInt32 i = readUnsigned(1);    // identification byte (should be 2)
-	if (i == 2)
-		while (readUnsigned(1) == 223);  // skip fill bytes (223), eof bit should be set now
-	else
-		throw DVIException("invalid identification value in postpost");
+	readUnsigned(4);   // pointer to begin of postamble
+	_dviFormat = max(_dviFormat, (DVIFormat)readUnsigned(1));  // identification byte
+	verifyDVIFormat(_dviFormat);
+	while (readUnsigned(1) == 223);  // skip fill bytes (223), eof bit should be set now
 }
 
 
@@ -393,9 +403,9 @@ void DVIReader::cmdBop (int) {
 	for (int i=0; i < 10; i++)
 		c[i] = readSigned(4);
 	readSigned(4);        // pointer to peceeding bop (-1 in case of first page)
-	_currPos.reset();     // set all DVI registers to 0
-	while (!_posStack.empty())
-		_posStack.pop();
+	_dviState.reset();     // set all DVI registers to 0
+	while (!_stateStack.empty())
+		_stateStack.pop();
 	_currFontNum = 0;
 	_inPage = true;
 	_pageLength = 0;
@@ -421,34 +431,36 @@ void DVIReader::cmdBop (int) {
 
 /** Reads and executes End-Of-Page command. */
 void DVIReader::cmdEop (int) {
-	if (!_posStack.empty())
+	if (!_stateStack.empty())
 		throw DVIException("stack not empty at end of page");
 	_inPage = false;
-	endPage();
+	endPage(_currPageNum);
 	if (_actions)
-		_actions->endPage();
+		_actions->endPage(_currPageNum);
 }
 
 
 /** Reads and executes push command. */
 void DVIReader::cmdPush (int) {
-	_posStack.push(_currPos);
+	_stateStack.push(_dviState);
 }
 
 
 /** Reads and executes pop command (restores pushed position information). */
 void DVIReader::cmdPop (int) {
-	if (_posStack.empty())
+	if (_stateStack.empty())
 		throw DVIException("stack empty at pop command");
 	else {
-		DVIPosition prevPos = _currPos;
-		_currPos = _posStack.top();
-		_posStack.pop();
+		DVIState prevState = _dviState;
+		_dviState = _stateStack.top();
+		_stateStack.pop();
 		if (_actions) {
-			if (prevPos.h != _currPos.h)
-				_actions->moveToX(_currPos.h + _tx);
-			if (prevPos.v != _currPos.v)
-				_actions->moveToY(_currPos.v + _ty);
+			if (prevState.h != _dviState.h)
+				_actions->moveToX(_dviState.h + _tx);
+			if (prevState.v != _dviState.v)
+				_actions->moveToY(_dviState.v + _ty);
+			if (prevState.d != _dviState.d)
+				_actions->setTextOrientation(_dviState.d != WMODE_LR);
 		}
 	}
 }
@@ -463,6 +475,9 @@ void DVIReader::putChar (UInt32 c, bool moveCursor) {
 	if (!_inPage)
 		throw DVIException("set_char or put_char outside of page");
 
+	if (_actions && !_actions->fontProcessingEnabled())
+		return;
+
 	FontManager &fm = FontManager::instance();
 	Font *font = fm.getFont(_currFontNum);
 	if (!font)
@@ -471,13 +486,14 @@ void DVIReader::putChar (UInt32 c, bool moveCursor) {
 	if (VirtualFont *vf = dynamic_cast<VirtualFont*>(font)) {    // is current font a virtual font?
 		vector<UInt8> *dvi = const_cast<vector<UInt8>*>(vf->getDVI(c)); // get DVI snippet that describes character c
 		if (dvi) {
-			DVIPosition pos = _currPos;       // save current cursor position
-			_currPos.x = _currPos.y = _currPos.w = _currPos.z = 0;
+			DVIState pos = _dviState;        // save current cursor position
+			_dviState.x = _dviState.y = _dviState.w = _dviState.z = 0;
 			int save_fontnum = _currFontNum; // save current font number
-			fm.enterVF(vf);        // new font number context
+			fm.enterVF(vf);                  // new font number context
 			cmdFontNum0(fm.vfFirstFontNum(vf));
-			double save_scale = _scaleFactor;
-			_scaleFactor = vf->scaledSize()/(1 << 20);
+			double save_scale = _dvi2bp;
+			// DVI units in virtual fonts are multiples of 1^(-20) times the scaled size of the VF
+			_dvi2bp = vf->scaledSize()/(1 << 20);
 
 			VectorInputStream<UInt8> vis(*dvi);
 			istream &is = replaceStream(vis);
@@ -488,17 +504,23 @@ void DVIReader::putChar (UInt32 c, bool moveCursor) {
 				//					Message::estream(true) << "invalid dvi in vf: " << e.getMessage() << endl; // @@
 			}
 			replaceStream(is);          // restore previous input stream
-			_scaleFactor = save_scale;  // restore previous scale factor
+			_dvi2bp = save_scale;  // restore previous scale factor
 			fm.leaveVF();               // restore previous font number context
 			cmdFontNum0(save_fontnum);  // restore previous font number
-			_currPos = pos;             // restore previous cursor position
+			_dviState = pos;            // restore previous cursor position
 		}
 	}
-	else if (_actions) {
-		_actions->setChar(_currPos.h+_tx, _currPos.v+_ty, c, font);
+	else if (_actions)
+		_actions->setChar(_dviState.h+_tx, _dviState.v+_ty, c, _dviState.d != WMODE_LR, font);
+
+	if (moveCursor) {
+		double dist = font->charWidth(c) * font->scaleFactor() * _mag/1000.0;
+		switch (_dviState.d) {
+			case WMODE_LR: _dviState.h += dist; break;
+			case WMODE_TB: _dviState.v += dist; break;
+			case WMODE_BT: _dviState.v -= dist; break;
+		}
 	}
-	if (moveCursor)
-		_currPos.h += font->charWidth(c) * font->scaleFactor() * _mag/1000.0;
 }
 
 
@@ -540,13 +562,11 @@ void DVIReader::cmdPutChar (int len) {
  *  @throw DVIException if method is called ouside a bop/eop pair */
 void DVIReader::cmdSetRule (int) {
 	if (_inPage) {
-		double height = _scaleFactor*readSigned(4);
-		double width  = _scaleFactor*readSigned(4);
+		double height = _dvi2bp*readSigned(4);
+		double width  = _dvi2bp*readSigned(4);
 		if (_actions && height > 0 && width > 0)
-			_actions->setRule(_currPos.h+_tx, _currPos.v+_ty, height, width);
-		_currPos.h += width;
-		if (_actions && (height <= 0 || width <= 0))
-			_actions->moveToX(_currPos.h+_tx);
+			_actions->setRule(_dviState.h+_tx, _dviState.v+_ty, height, width);
+		moveRight(width);
 	}
 	else
 		throw DVIException("set_rule outside of page");
@@ -558,28 +578,74 @@ void DVIReader::cmdSetRule (int) {
  *  @throw DVIException if method is called ouside a bop/eop pair */
 void DVIReader::cmdPutRule (int) {
 	if (_inPage) {
-		double height = _scaleFactor*readSigned(4);
-		double width  = _scaleFactor*readSigned(4);
+		double height = _dvi2bp*readSigned(4);
+		double width  = _dvi2bp*readSigned(4);
 		if (_actions && height > 0 && width > 0)
-			_actions->setRule(_currPos.h+_tx, _currPos.v+_ty, height, width);
+			_actions->setRule(_dviState.h+_tx, _dviState.v+_ty, height, width);
 	}
 	else
 		throw DVIException("put_rule outside of page");
 }
 
 
+void DVIReader::moveRight (double dx) {
+	switch (_dviState.d) {
+		case WMODE_LR: _dviState.h += dx; break;
+		case WMODE_TB: _dviState.v += dx; break;
+		case WMODE_BT: _dviState.v -= dx; break;
+	}
+	if (_actions) {
+		if (_dviState.d == WMODE_LR)
+			_actions->moveToX(_dviState.h+_tx);
+		else
+			_actions->moveToY(_dviState.v+_ty);
+	}
+}
+
+
+void DVIReader::moveDown (double dy) {
+	switch (_dviState.d) {
+		case WMODE_LR: _dviState.v += dy; break;
+		case WMODE_TB: _dviState.h -= dy; break;
+		case WMODE_BT: _dviState.h += dy; break;
+	}
+	if (_actions) {
+		if (_dviState.d == WMODE_LR)
+			_actions->moveToY(_dviState.v+_ty);
+		else
+			_actions->moveToX(_dviState.h+_tx);
+	}
+}
+
+
+void DVIReader::cmdRight (int len) {moveRight(_dvi2bp*readSigned(len));}
+void DVIReader::cmdDown (int len)  {moveDown(_dvi2bp*readSigned(len));}
+void DVIReader::cmdX0 (int)        {moveRight(_dviState.x);}
+void DVIReader::cmdY0 (int)        {moveDown(_dviState.y);}
+void DVIReader::cmdW0 (int)        {moveRight(_dviState.w);}
+void DVIReader::cmdZ0 (int)        {moveDown(_dviState.z);}
+void DVIReader::cmdX (int len)     {_dviState.x = _dvi2bp*readSigned(len); cmdX0(0);}
+void DVIReader::cmdY (int len)     {_dviState.y = _dvi2bp*readSigned(len); cmdY0(0);}
+void DVIReader::cmdW (int len)     {_dviState.w = _dvi2bp*readSigned(len); cmdW0(0);}
+void DVIReader::cmdZ (int len)     {_dviState.z = _dvi2bp*readSigned(len); cmdZ0(0);}
 void DVIReader::cmdNop (int)       {}
 
-void DVIReader::cmdRight (int len) {_currPos.h += _scaleFactor*readSigned(len); if (_actions) _actions->moveToX(_currPos.h+_tx);}
-void DVIReader::cmdDown (int len)  {_currPos.v += _scaleFactor*readSigned(len); if (_actions) _actions->moveToY(_currPos.v+_ty);}
-void DVIReader::cmdX0 (int)        {_currPos.h += _currPos.x;                   if (_actions) _actions->moveToX(_currPos.h+_tx);}
-void DVIReader::cmdY0 (int)        {_currPos.v += _currPos.y;                   if (_actions) _actions->moveToY(_currPos.v+_ty);}
-void DVIReader::cmdW0 (int)        {_currPos.h += _currPos.w;                   if (_actions) _actions->moveToX(_currPos.h+_tx);}
-void DVIReader::cmdZ0 (int)        {_currPos.v += _currPos.z;                   if (_actions) _actions->moveToY(_currPos.v+_ty);}
-void DVIReader::cmdX (int len)     {_currPos.x = _scaleFactor*readSigned(len);  cmdX0(0);}
-void DVIReader::cmdY (int len)     {_currPos.y = _scaleFactor*readSigned(len);  cmdY0(0);}
-void DVIReader::cmdW (int len)     {_currPos.w = _scaleFactor*readSigned(len);  cmdW0(0);}
-void DVIReader::cmdZ (int len)     {_currPos.z = _scaleFactor*readSigned(len);  cmdZ0(0);}
+
+/** Sets the text orientation (horizontal, vertical).
+ *  This command is only available in DVI files of format 3 (created by pTeX) */
+void DVIReader::cmdDir (int) {
+	UInt8 wmode = readUnsigned(1);
+	if (wmode == 4)  // yoko mode (4) equals default LR mode (0)
+		wmode = 0;
+	if (wmode == 2 || wmode > 3) {
+		ostringstream oss;
+		oss << "invalid writing mode value " << wmode << " (0, 1, or 3 expected)";
+		throw DVIException(oss.str());
+	}
+	_dviState.d = (WritingMode)wmode;
+	if (_actions)
+		_actions->setTextOrientation(_dviState.d != WMODE_LR);
+}
 
 
 void DVIReader::cmdXXX (int len) {
@@ -587,7 +653,7 @@ void DVIReader::cmdXXX (int len) {
 		UInt32 numBytes = readUnsigned(len);
 		string s = readString(numBytes);
 		if (_actions)
-			_actions->special(s);
+			_actions->special(s, _dvi2bp);
 	}
 	else
 		throw DVIException("special outside of page");
@@ -598,6 +664,9 @@ void DVIReader::cmdXXX (int len) {
  * @param[in] num font number
  * @throw DVIException if font number is undefined */
 void DVIReader::cmdFontNum0 (int num) {
+	if (_actions && !_actions->fontProcessingEnabled())
+		return;
+
 	if (Font *font = FontManager::instance().getFont(num)) {
 		_currFontNum = num;
 		if (_actions && !dynamic_cast<VirtualFont*>(font))
@@ -623,11 +692,13 @@ void DVIReader::cmdFontNum (int len) {
 /** Helper function to handle a font definition.
  *  @param[in] fontnum local font number
  *  @param[in] name font name
- *  @param[in] checksum checksum to be compared with TFM checksum
- *  @param[in] ds design size in TeX point units
- *  @param[in] ss scaled size in TeX point units */
+ *  @param[in] cs checksum to be compared with TFM checksum
+ *  @param[in] ds design size in PS point units
+ *  @param[in] ss scaled size in PS point units */
 void DVIReader::defineFont (UInt32 fontnum, const string &name, UInt32 cs, double ds, double ss) {
-   FontManager &fm = FontManager::instance();
+	if (_actions && !_actions->fontProcessingEnabled())
+		return;
+	FontManager &fm = FontManager::instance();
 	int id = fm.registerFont(fontnum, name, cs, ds, ss);
 	Font *font = fm.getFontById(id);
 	if (VirtualFont *vf = dynamic_cast<VirtualFont*>(font)) {
@@ -640,7 +711,7 @@ void DVIReader::defineFont (UInt32 fontnum, const string &name, UInt32 cs, doubl
 		fm.leaveVF();
 	}
 	if (_actions)
-		_actions->defineFont(id, font);
+	_actions->defineFont(id, font);
 }
 
 
@@ -656,21 +727,21 @@ void DVIReader::cmdFontDef (int len) {
 	string fontpath = readString(pathlen);
 	string fontname = readString(namelen);
 
-	// process font definitions between bop and eop only, skip those in the postamble
-	if (_inPage)
-		defineFont(fontnum, fontname, checksum, dsize*_scaleFactor, ssize*_scaleFactor);
+	// @@ TODO: avoid processing the font definitions in the postamble
+	defineFont(fontnum, fontname, checksum, dsize*_dvi2bp, ssize*_dvi2bp);
 }
 
 
 /** This template method is called by the VFReader after reading a font definition from a VF file.
  *  @param[in] fontnum local font number
+ *  @param[in] path path to font file
  *  @param[in] name font name
  *  @param[in] checksum checksum to be compared with TFM checksum
- *  @param[in] dsize design size in TeX point units
- *  @param[in] ssize scaled size in TeX point units */
+ *  @param[in] dsize design size in PS point units
+ *  @param[in] ssize scaled size in PS point units */
 void DVIReader::defineVFFont (UInt32 fontnum, string path, string name, UInt32 checksum, double dsize, double ssize) {
-	VirtualFont *vf = FontManager::instance().getVF();
-	defineFont(fontnum, name, checksum, dsize, ssize * vf->scaleFactor());
+	if (VirtualFont *vf = FontManager::instance().getVF())
+		defineFont(fontnum, name, checksum, dsize, ssize * vf->scaleFactor());
 }
 
 
@@ -680,3 +751,90 @@ void DVIReader::defineVFFont (UInt32 fontnum, string path, string name, UInt32 c
 void DVIReader::defineVFChar (UInt32 c, vector<UInt8> *dvi) {
 	FontManager::instance().assignVfChar(c, dvi);
 }
+
+
+/** XDV extension: include image or pdf file.
+ *  parameters: box[1] matrix[4][6] p[2] len[2] path[l] */
+void DVIReader::cmdXPic (int) {
+	// just skip the parameters
+	readUnsigned(1);           // box
+	for (int i=0; i < 6; i++)  // matrix
+		readSigned(4);
+	readSigned(2);             // page number
+	UInt16 len = readUnsigned(2);
+	readString(len);           // path to image/pdf file
+}
+
+
+void DVIReader::cmdXFontDef (int) {
+	Int32 fontnum = readSigned(4);
+	double ptsize = _dvi2bp*readUnsigned(4);
+	UInt16 flags = readUnsigned(2);
+	UInt8 psname_len = readUnsigned(1);
+	UInt8 fmname_len = readUnsigned(1);
+	UInt8 stname_len = readUnsigned(1);
+	string fontname = readString(psname_len);
+	readString(fmname_len);
+	readString(stname_len);
+	FontStyle style;
+	Color color;
+	if (flags & 0x0100) { // vertical?
+	}
+	if (flags & 0x0200) { // colored?
+		// The font color must not interfere with color specials. If the font color is not black,
+		// all color specials should be ignored, i.e. glyphs of a non-black font have a fixed color
+		// that can't be changed by colors specials.
+		UInt32 rgba = readUnsigned(4);
+		color.set(UInt8(rgba >> 24), UInt8((rgba >> 16) & 0xff), UInt8((rgba >> 8) & 0xff));
+	}
+	if (flags & 0x1000)   // extend?
+		style.extend = _dvi2bp*readSigned(4);
+	if (flags & 0x2000)   // slant?
+		style.slant = _dvi2bp*readSigned(4);
+	if (flags & 0x4000)   // embolden?
+		style.bold = _dvi2bp*readSigned(4);
+	if (flags & 0x0800) { // variations?
+		UInt16 num_variations = readSigned(2);
+		for (int i=0; i < num_variations; i++)
+			readUnsigned(4);
+	}
+	if (_inPage && _actions && _actions->fontProcessingEnabled())
+		FontManager::instance().registerFont(fontnum, fontname, ptsize, style, color);
+}
+
+
+void DVIReader::cmdXGlyphA (int) {
+	putGlyphArray(false);
+}
+
+
+void DVIReader::cmdXGlyphS (int) {
+	putGlyphArray(true);
+}
+
+
+void DVIReader::putGlyphArray (bool xonly) {
+	double strwidth = _dvi2bp*readSigned(4);
+	UInt16 num_glyphs = readUnsigned(2);
+	vector<Int32> x(num_glyphs);
+	vector<Int32> y(num_glyphs);
+	for (int i=0; i < num_glyphs; i++) {
+		x[i] = readSigned(4);
+		y[i] = xonly ? 0 : readSigned(4);
+	}
+	if (_actions && _actions->fontProcessingEnabled()) {
+		if (Font *font = FontManager::instance().getFont(_currFontNum)) {
+			for (int i=0; i < num_glyphs; i++) {
+				UInt16 glyph_index = readUnsigned(2);
+				double xx = _dviState.h + x[i]*_dvi2bp + _tx;
+				double yy = _dviState.v + y[i]*_dvi2bp + _ty;
+				_actions->setChar(xx, yy, glyph_index, false, font);
+			}
+		}
+	}
+	else {
+		for (int i=0; i < num_glyphs; i++)
+			readUnsigned(2);
+	}
+	moveRight(strwidth);
+}
