@@ -2,7 +2,7 @@
 ** DVIToSVG.cpp                                                         **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2013 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,17 +18,7 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#ifdef HAVE_CONFIG_H
-	#include "config.h"
-	#ifdef TARGET_SYSTEM
-		#define VERSION_STR VERSION " (" TARGET_SYSTEM ")"
-	#else
-		#define VERSION_STR VERSION
-	#endif
-#else
-	#define VERSION_STR ""
-#endif
-
+#include <config.h>
 #include <cstdlib>
 #include <ctime>
 #include <fstream>
@@ -47,12 +37,6 @@
 #include "PageRanges.h"
 #include "PageSize.h"
 #include "SVGOutputBase.h"
-#include "SVGTree.h"
-#include "TFM.h"
-#include "XMLDocument.h"
-#include "XMLDocTypeNode.h"
-#include "XMLNode.h"
-#include "XMLString.h"
 //
 ///////////////////////////////////
 // special handlers
@@ -61,38 +45,39 @@
 #include "ColorSpecialHandler.h"
 #include "DvisvgmSpecialHandler.h"
 #include "EmSpecialHandler.h"
-#include "NoPsSpecialHandler.h"
 #include "PdfSpecialHandler.h"
-//#include "HtmlSpecialHandler.h"
-#if !defined(DISABLE_GS)
+#include "HtmlSpecialHandler.h"
+#ifndef HAVE_LIBGS
+	#include "NoPsSpecialHandler.h"
+#endif
+#ifndef DISABLE_GS
 	#include "PsSpecialHandler.h"
 #endif
 #include "TpicSpecialHandler.h"
+
 ///////////////////////////////////
 
 
 using namespace std;
 
 
-/** Returns time stamp of current date/time. */
-static string datetime () {
-	time_t t;
-	time(&t);
-	struct tm *tm = localtime(&t);
-	char *timestr = asctime(tm);
-	timestr[24] = 0;  // remove newline
-	return timestr;
-}
-
 class PSHeaderActions : public DVIActions
 {
 	public :
 		PSHeaderActions (DVIToSVG &dvisvg) : _dvisvg(dvisvg) {}
 
-		void special (const std::string &s) {
+		void special (const std::string &str, double dvi2bp) {
 			// execute PS headers only
-			if (!s.empty() && (s[0] == '!' || s.substr(0, 7) == "header="))
-				_dvisvg.specialManager().process(s, 0);
+			if (!str.empty() && (str[0] == '!' || str.substr(0, 7) == "header="))
+				_dvisvg.specialManager().process(str, dvi2bp, 0);
+		}
+
+		bool fontProcessingEnabled () const {
+			return false;
+		}
+
+		void endPage (unsigned) {
+			_dvisvg.specialManager().leavePSHeaderSection();
 		}
 
 		BoundingBox& bbox () {return _bbox;}
@@ -140,14 +125,21 @@ void DVIToSVG::convert (unsigned first, unsigned last, pair<int,int> *pageinfo)
 	}
 	last = min(last, getTotalPages());
 
-	if (first > 1) {
-		// ensure loading of PostScript prologues given at the beginning of the first page
-		// (prologue files are always referenced in first page)
-		PSHeaderActions headerActions(*this);
-		DVIActions *save = replaceActions(&headerActions);
+#ifndef DISABLE_GS
+	// ensure loading of PostScript prologues given at the beginning of the first page
+	// (prologue files are always referenced in first page)
+	PSHeaderActions headerActions(*this);
+	DVIActions *save = replaceActions(&headerActions);
+	try {
 		executePage(1);
 		replaceActions(save);
 	}
+	catch (PSException &e) {
+		replaceActions(save);
+		throw;
+	}
+#endif
+
 	for (unsigned i=first; i <= last; ++i) {
 		executePage(i);
 		embedFonts(_svg.rootNode());
@@ -186,42 +178,35 @@ void DVIToSVG::convert (const string &rangestr, pair<int,int> *pageinfo) {
 
 /** This template method is called by parent class DVIReader before
  *  executing the BOP actions.
+ *  @param[in] pageno physical page number (1 = first page)
  *  @param[in] c contains information about the page (page number etc.) */
-void DVIToSVG::beginPage (unsigned n, Int32 *c) {
+void DVIToSVG::beginPage (unsigned pageno, Int32 *c) {
 	if (dynamic_cast<DVIToSVGActions*>(getActions())) {
 		Message::mstream().indent(0);
-		Message::mstream(false, Message::MC_PAGE_NUMBER) << "processing page " << n << " [" << c[0] << ']';
+		Message::mstream(false, Message::MC_PAGE_NUMBER) << "processing page " << pageno;
+		if (pageno != (unsigned)c[0])  // Does page number shown on page differ from physical page number?
+			Message::mstream(false) << " [" << c[0] << ']';
 		Message::mstream().indent(1);
-		_svg.appendToDoc(new XMLCommentNode(" This file was generated by dvisvgm " VERSION_STR " "));
-		_svg.appendToDoc(new XMLCommentNode(" " + datetime() + " "));
+		_svg.appendToDoc(new XMLCommentNode(" This file was generated by dvisvgm " VERSION " "));
 	}
 }
 
 
 /** This template method is called by parent class DVIReader before
  *  executing the EOP actions. */
-void DVIToSVG::endPage () {
+void DVIToSVG::endPage (unsigned pageno) {
 	if (!dynamic_cast<DVIToSVGActions*>(getActions()))
 		return;
 
-	_specialManager.notifyEndPage();
+	_specialManager.notifyEndPage(pageno);
 	// set bounding box and apply page transformations
 	BoundingBox &bbox = getActions()->bbox();
-	if (!_transCmds.empty()) {
-		Calculator calc;
-		calc.setVariable("ux", bbox.minX());
-		calc.setVariable("uy", bbox.minY());
-		calc.setVariable("w", bbox.width());
-		calc.setVariable("h", bbox.height());
-		calc.setVariable("pt", 1);
-		calc.setVariable("in", 72.27);
-		calc.setVariable("cm", 72.27/2.54);
-		calc.setVariable("mm", 72.27/25.4);
-		Matrix matrix(_transCmds, calc);
-		static_cast<DVIToSVGActions*>(getActions())->setPageMatrix(matrix);
-		if (_bboxString == "min")
-			bbox.transform(matrix);
-	}
+	bbox.unlock();
+	Matrix matrix;
+	getPageTransformation(matrix);
+	static_cast<DVIToSVGActions*>(getActions())->setPageMatrix(matrix);
+	if (_bboxString == "min")
+		bbox.transform(matrix);
 	if (string("dvi none min").find(_bboxString) == string::npos) {
 		istringstream iss(_bboxString);
 		StreamInputReader ir(iss);
@@ -232,8 +217,8 @@ void DVIToSVG::endPage () {
 			if (size.valid()) {
 				// convention: DVI position (0,0) equals (1in, 1in) relative
 				// to the upper left vertex of the page (see DVI specification)
-				const double border = -72.27;
-				bbox = BoundingBox(border, border, size.widthInPT()+border, size.heightInPT()+border);
+				const double border = -72;
+				bbox = BoundingBox(border, border, size.widthInBP()+border, size.heightInBP()+border);
 			}
 		}
 		else { // set/modify bounding box by explicitly given values
@@ -256,17 +241,41 @@ void DVIToSVG::endPage () {
 		else {
 			_svg.setBBox(bbox);
 
+			const double bp2pt = 72.27/72;
+			const double bp2mm = 25.4/72;
 			Message::mstream(false) << '\n';
-			Message::mstream(false, Message::MC_PAGE_SIZE) << "page size: " << bbox.width() << "pt"
-				" x " << bbox.height() << "pt"
-				" (" << bbox.width()/72.27*25.4 << "mm"
-				" x " << bbox.height()/72.27*25.4 << "mm)";
+			Message::mstream(false, Message::MC_PAGE_SIZE) << "page size: " << XMLString(bbox.width()*bp2pt) << "pt"
+				" x " << XMLString(bbox.height()*bp2pt) << "pt"
+				" (" << XMLString(bbox.width()*bp2mm) << "mm"
+				" x " << XMLString(bbox.height()*bp2mm) << "mm)";
 			Message::mstream(false) << '\n';
 		}
 	}
 }
 
 
+void DVIToSVG::getPageTransformation(Matrix &matrix) const {
+	if (_transCmds.empty())
+		matrix.set(1);  // unity matrix
+	else {
+		Calculator calc;
+		if (getActions()) {
+			const double bp2pt = 72.27/72;
+			BoundingBox &bbox = getActions()->bbox();
+			calc.setVariable("ux", bbox.minX()*bp2pt);
+			calc.setVariable("uy", bbox.minY()*bp2pt);
+			calc.setVariable("w",  bbox.width()*bp2pt);
+			calc.setVariable("h",  bbox.height()*bp2pt);
+		}
+		calc.setVariable("pt", 1);
+		calc.setVariable("in", 72.27);
+		calc.setVariable("cm", 72.27/2.54);
+		calc.setVariable("mm", 72.27/25.4);
+		matrix.set(_transCmds, calc);
+	}
+}
+
+
 static void collect_chars (map<const Font*, set<int> > &fm) {
 	typedef const map<const Font*, set<int> > UsedCharsMap;
 	FORALL(fm, UsedCharsMap::const_iterator, it) {
@@ -331,26 +340,28 @@ const SpecialManager* DVIToSVG::setProcessSpecials (const char *ignorelist, bool
 			new ColorSpecialHandler,    // handles color specials
 			new DvisvgmSpecialHandler,  // handles raw SVG embeddings
 			new EmSpecialHandler,       // handles emTeX specials
+			new HtmlSpecialHandler,     // handles hyperref specials
 			new PdfSpecialHandler,      // handles pdf specials
-//			new HtmlSpecialHandler,     // handles hyperref specials
 			new TpicSpecialHandler,     // handles tpic specials
 			0
 		};
 		SpecialHandler **p = handlers;
-#if !defined(DISABLE_GS)
+#ifndef DISABLE_GS
 		if (Ghostscript().available())
 			*p = new PsSpecialHandler;
 		else
 #endif
 		{
+#ifndef HAVE_LIBGS
 			*p = new NoPsSpecialHandler; // dummy PS special handler that only prints warning messages
 			if (pswarning) {
-#if defined(DISABLE_GS)
-				Message::wstream() << "this build of dvisvgm doesn't support the processing of PostScript specials\n";
+#ifdef DISABLE_GS
+				Message::wstream() << "processing of PostScript specials has been disabled permanently\n";
 #else
 				Message::wstream() << "processing of PostScript specials is disabled (Ghostscript not found)\n";
 #endif
 			}
+#endif
 		}
 		_specialManager.unregisterHandlers();
 		_specialManager.registerHandlers(p, ignorelist);
