@@ -2,7 +2,7 @@
 ** DVIToSVGActions.cpp                                                  **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2013 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,6 +18,7 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
+#include <config.h>
 #include <cstring>
 #include <ctime>
 #include "BoundingBox.h"
@@ -31,11 +32,11 @@
 #include "XMLNode.h"
 #include "XMLString.h"
 
+
 using namespace std;
 
 
 double DVIToSVGActions::PROGRESSBAR_DELAY=1000;  // initial delay in seconds (values >= 1000 disable the progressbar)
-bool DVIToSVGActions::EXACT_BBOX = false;
 
 
 DVIToSVGActions::DVIToSVGActions (DVIToSVG &dvisvg, SVGTree &svg)
@@ -69,7 +70,7 @@ void DVIToSVGActions::setPageMatrix (const Matrix &matrix) {
 
 void DVIToSVGActions::moveToX (double x) {
 	_dvisvg.specialManager().notifyPositionChange(getX(), getY());
-	_svg.setX(getX());        // get current horizontal DVI position
+	_svg.setX(x);
 }
 
 
@@ -84,8 +85,9 @@ void DVIToSVGActions::moveToY (double y) {
  *  @param[in] x horizontal position of left bounding box edge
  *  @param[in] y vertical position of the character's baseline
  *  @param[in] c character code relative to the current font
+ *  @param[in] vertical true if we're in vertical mode
  *  @param[in] font font to be used */
-void DVIToSVGActions::setChar (double x, double y, unsigned c, const Font *font) {
+void DVIToSVGActions::setChar (double x, double y, unsigned c, bool vertical, const Font *font) {
 	// If we use SVG fonts there is no need to record all font name/char/size combinations
 	// because the SVG font mechanism handles this automatically. It's sufficient to
 	// record font names and chars. The various font sizes can be ignored here.
@@ -95,41 +97,62 @@ void DVIToSVGActions::setChar (double x, double y, unsigned c, const Font *font)
 
 	// However, we record all required fonts
 	_usedFonts.insert(font);
-
 	_svg.appendChar(c, x, y, *font);
-	// update bounding box
-	double wl=0, wr=0, h=0, d=0; // left/right width, height, and depth of character c
+
 	static string fontname;
 	GlyphTracerMessages callback(fontname != font->name(), false);
 	fontname = font->name();
-	BoundingBox charbox;
-	const PhysicalFont *ph_font = dynamic_cast<const PhysicalFont*>(font);
-	if (EXACT_BBOX && ph_font && ph_font->getGlyphBox(c, charbox, &callback)) {
-		if ((wl = charbox.minX()) > 0) wl=0;
-		if ((wr = charbox.maxX()) < 0) wr=0;
-		if ((h = charbox.maxY()) < 0) h=0;
-		if ((d = -charbox.minY()) < 0) d=0;
-	}
-	else {
-		double s = font->scaleFactor();
-		wr = s*(font->charWidth(c) + font->italicCorr(c));
-		h  = s*font->charHeight(c);
-		d  = s*font->charDepth(c);
-	}
 
-	BoundingBox bbox(x+wl, y-h, x+wr, y+d);
-/*	XMLElementNode *rect = new XMLElementNode("rect");
-	rect->addAttribute("x", x+wl);
-	rect->addAttribute("y", y-h);
-	rect->addAttribute("width", (-wl+wr));
-	rect->addAttribute("height", (h+d));
-	rect->addAttribute("fill", "none");
-	rect->addAttribute("stroke", "red");
-	rect->addAttribute("stroke-width", "0.5");
-	_svg.appendToPage(rect);*/
+	BoundingBox charbox;
+	GlyphMetrics metrics;
+	font->getGlyphMetrics(c, vertical, metrics);
+	const PhysicalFont* pf = dynamic_cast<const PhysicalFont*>(font);
+	if (PhysicalFont::EXACT_BBOX && pf) {
+		GlyphMetrics exact_metrics;
+		pf->getExactGlyphBox(c, exact_metrics, vertical, &callback);
+		if (vertical) {
+			// move top of bbox to upper margin of glyph (just an approximation yet)
+			y += (metrics.d-exact_metrics.h-exact_metrics.d)/2;
+		}
+		metrics = exact_metrics;
+	}
+	BoundingBox bbox(x-metrics.wl, y-metrics.h, x+metrics.wr, y+metrics.d);
+
+	// update bounding box
 	if (!getMatrix().isIdentity())
 		bbox.transform(getMatrix());
 	embed(bbox);
+#if 0
+	XMLElementNode *rect = new XMLElementNode("rect");
+	rect->addAttribute("x", x-metrics.wl);
+	rect->addAttribute("y", y-metrics.h);
+	rect->addAttribute("width", metrics.wl+metrics.wr);
+	rect->addAttribute("height", metrics.h+metrics.d);
+	rect->addAttribute("fill", "none");
+	rect->addAttribute("stroke", "red");
+	rect->addAttribute("stroke-width", "0.5");
+	_svg.appendToPage(rect);
+	if (metrics.d > 0) {
+		XMLElementNode *line = new XMLElementNode("line");
+		line->addAttribute("x1", x-metrics.wl);
+		line->addAttribute("y1", y);
+		line->addAttribute("x2", x+metrics.wr);
+		line->addAttribute("y2", y);
+		line->addAttribute("stroke", "blue");
+		line->addAttribute("stroke-width", "0.5");
+		_svg.appendToPage(line);
+	}
+	if (metrics.wl > 0) {
+		XMLElementNode *line = new XMLElementNode("line");
+		line->addAttribute("x1", x);
+		line->addAttribute("y1", y-metrics.h);
+		line->addAttribute("x2", x);
+		line->addAttribute("y2", y+metrics.d);
+		line->addAttribute("stroke", "blue");
+		line->addAttribute("stroke-width", "0.5");
+		_svg.appendToPage(line);
+	}
+#endif
 }
 
 
@@ -140,10 +163,6 @@ void DVIToSVGActions::setChar (double x, double y, unsigned c, const Font *font)
  *  @param[in] height length of the vertical edges
  *  @param[in] width length of the horizontal edges */
 void DVIToSVGActions::setRule (double x, double y, double height, double width) {
-/*	x *= BP;
-	y *= BP;
-	height *= BP;
-	width  *= BP; */
 	// (x,y) is the lower left corner of the rectangle
 	XMLElementNode *rect = new XMLElementNode("rect");
 	rect->addAttribute("x", x);
@@ -175,10 +194,11 @@ void DVIToSVGActions::setFont (int num, const Font *font) {
 
 
 /** This method is called when a "special" command was found in the DVI file.
- *  @param[in] s the special expression */
-void DVIToSVGActions::special (const string &s) {
+ *  @param[in] s the special expression
+ *  @param[in] dvi2bp factor to scale DVI units to PS points */
+void DVIToSVGActions::special (const string &s, double dvi2bp) {
 	try {
-		_dvisvg.specialManager().process(s, this, this);
+		_dvisvg.specialManager().process(s, dvi2bp, this, this);
 		// @@ output message in case of unsupported specials?
 	}
 	catch (const SpecialException &e) {
@@ -206,9 +226,10 @@ void DVIToSVGActions::postamble () {
 
 
 /** This method is called when a "begin of page (bop)" command was found in the DVI file.
- *  @param[in] c array with 10 components representing \count0 ... \count9. c[0] contains the
+ *  @param[in] pageno physical page number
+ *  @param[in] c array with 10 components representing \\count0 ... \\count9. c[0] contains the
  *               current (printed) page number (may differ from page count) */
-void DVIToSVGActions::beginPage (unsigned n, Int32 *c) {
+void DVIToSVGActions::beginPage (unsigned pageno, Int32 *c) {
 	_svg.newPage(++_pageCount);
 	_bbox = BoundingBox();  // clear bounding box
 	if (_boxes)
@@ -217,7 +238,7 @@ void DVIToSVGActions::beginPage (unsigned n, Int32 *c) {
 
 
 /** This method is called when an "end of page (eop)" command was found in the DVI file. */
-void DVIToSVGActions::endPage () {
+void DVIToSVGActions::endPage (unsigned pageno) {
 	_svg.transformPage(_pageMatrix);
 	if (_bgcolor != Color::TRANSPARENT) {
 		// create a rectangle filled with the background color
@@ -307,8 +328,10 @@ void DVIToSVGActions::progress (size_t current, size_t total, const char *id) {
 		Message::mstream() << '\n';
 	}
 	// don't show the progress indicator before the given time has elapsed
-	if (!draw && System::time()-time > PROGRESSBAR_DELAY)
+	if (!draw && System::time()-time > PROGRESSBAR_DELAY) {
 		draw = true;
+		Terminal::cursor(false);
+	}
 	if (draw && (System::time() - time > 0.1 || (total > 0 && current == total) || prev_id != id)) {
 		static int step = -1;   // >=0: rotating dash
 		static size_t prev_current=0, prev_total=1;
@@ -335,8 +358,10 @@ void DVIToSVGActions::progress (size_t current, size_t total, const char *id) {
 			<< "] " << string(3-digits(int(100*factor)), ' ') << int(100*factor)
 			<< "%\r";
 		// overprint indicator when finished
-		if (factor == 1.0)
+		if (factor == 1.0) {
 			Message::estream().clearline();
+			Terminal::cursor(true);
+		}
 		time = System::time();
 		prev_id = id;
 	}
