@@ -2,7 +2,7 @@
 ** FontMap.cpp                                                          **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2013 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,11 +18,13 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
+#include <config.h>
 #include <cstring>
 #include <fstream>
 #include <iostream>
 #include <limits>
 #include <vector>
+#include "CMap.h"
 #include "Directory.h"
 #include "FileFinder.h"
 #include "FontManager.h"
@@ -58,7 +60,8 @@ bool FontMap::read (const string &fname, FontMap::Mode mode) {
 
 	int line_number = 1;
 	while (ifs) {
-		if (strchr("\n&#%;*", ifs.peek()))  // comment line?
+		int c = ifs.peek();
+		if (c < 0 || strchr("\n&#%;*", c))  // comment or empty line?
 			ifs.ignore(numeric_limits<int>::max(), '\n');
 		else {
 			try {
@@ -243,10 +246,10 @@ ostream& FontMap::write (ostream &os) const {
 
 
 /** Reads and evaluates all map files in the given directory.
- *  @param[in] path to directory containing the map files to be read */
+ *  @param[in] dirname path to directory containing the map files to be read */
 void FontMap::readdir (const string &dirname) {
 	Directory dir(dirname);
-	while (const char *fname = dir.read('f')) {
+	while (const char *fname = dir.read(Directory::ET_FILE)) {
 		if (strlen(fname) >= 4 && strcmp(fname+strlen(fname)-4, ".map") == 0) {
 			string path = dirname + "/" + fname;
 			read(path.c_str());
@@ -296,7 +299,7 @@ void FontMap::clear (bool unlocked_only) {
 /////////////////////////////////////////////////
 
 FontMap::Entry::Entry (const MapLine &mapline, Subfont *sf)
-	: fontname(mapline.fontfname()), encname(mapline.encname()), subfont(sf), fontindex(mapline.fontindex()), locked(false),
-	  bold(mapline.bold()), slant(mapline.slant()), extend(mapline.extend())
+	: fontname(mapline.fontfname()), encname(mapline.encname()), subfont(sf), fontindex(mapline.fontindex()),
+	  locked(false), style(mapline.bold(), mapline.extend(), mapline.slant())
 {
 }
