@@ -2,7 +2,7 @@
 ** Matrix.cpp                                                           **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2013 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,6 +18,8 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
+#include <config.h>
+#include <algorithm>
 #include <cmath>
 #include <limits>
 #include <sstream>
@@ -66,9 +68,7 @@ static inline double deg2rad (double deg) {
 /** Creates a diagonal matrix ((d,0,0),(0,d,0),(0,0,d)).
  *  @param[in] d value of diagonal elements */
 Matrix::Matrix (double d) {
-	for (int i=0; i < 3; i++)
-		for (int j=0; j < 3; j++)
-			_values[i][j] = (i==j ? d : 0);
+	set(d);
 }
 
 
@@ -97,6 +97,14 @@ Matrix::Matrix (const string &cmds, Calculator &calc) {
 }
 
 
+Matrix& Matrix::set (double d) {
+	for (int i=0; i < 3; i++)
+		for (int j=0; j < 3; j++)
+			_values[i][j] = (i==j ? d : 0);
+	return *this;
+}
+
+
 Matrix& Matrix::set (double v[], unsigned size) {
 	size = min(size, 9u);
 	for (unsigned i=0; i < size; i++)
@@ -122,6 +130,12 @@ Matrix& Matrix::set (const vector<double> &v, int start) {
 }
 
 
+Matrix& Matrix::set(const string &cmds, Calculator &calc) {
+	parse(cmds, calc);
+	return *this;
+}
+
+
 Matrix& Matrix::translate (double tx, double ty) {
 	if (tx != 0 || ty != 0) {
 		TranslationMatrix t(tx, ty);
@@ -150,10 +164,16 @@ Matrix& Matrix::rotate (double deg) {
 }
 
 
-Matrix& Matrix::xskew (double deg) {
-	double t = tan(deg2rad(deg));
-	if (t != 0) {
-		double v[] = {1, t};
+Matrix& Matrix::xskewByAngle (double deg) {
+	if (fmod(fabs(deg)-90, 180) != 0)
+		return xskewByRatio(tan(deg2rad(deg)));
+	return *this;
+}
+
+
+Matrix& Matrix::xskewByRatio (double xyratio) {
+	if (xyratio != 0) {
+		double v[] = {1, xyratio};
 		Matrix t(v, 2);
 		rmultiply(t);
 	}
@@ -161,10 +181,16 @@ Matrix& Matrix::xskew (double deg) {
 }
 
 
-Matrix& Matrix::yskew (double deg) {
-	double t = tan(deg2rad(deg));
-	if (t != 0) {
-		double v[] = {1, 0, 0, t};
+Matrix& Matrix::yskewByAngle (double deg) {
+	if (fmod(fabs(deg)-90, 180) != 0)
+		return yskewByRatio(tan(deg2rad(deg)));
+	return *this;
+}
+
+
+Matrix& Matrix::yskewByRatio (double xyratio) {
+	if (xyratio != 0) {
+		double v[] = {1, 0, 0, xyratio};
 		Matrix t(v, 4);
 		rmultiply(t);
 	}
@@ -316,7 +342,7 @@ static double getArgument (istream &is, Calculator &calc, double def, bool optio
    }
 	string expr;
 	while (is && !isupper(is.peek()) && is.peek() != ',')
-		expr += is.get();
+		expr += (char)is.get();
    if (expr.length() == 0) {
       if (optional)
          return def;
@@ -332,7 +358,7 @@ Matrix& Matrix::parse (istream &is, Calculator &calc) {
 	while (is) {
       while (isspace(is.peek()))
          is.get();
-		char cmd = is.get();
+		int cmd = is.get();
 		switch (cmd) {
 			case 'T': {
 				double tx = getArgument(is, calc, 0, false, false);
@@ -356,7 +382,7 @@ Matrix& Matrix::parse (istream &is, Calculator &calc) {
             break;
 			}
 			case 'F': {
-				char c = is.get();
+				int c = is.get();
 				if (c != 'H' && c != 'V')
 					throw ParserException("'H' or 'V' expected");
 				double a = getArgument(is, calc, 0, false, false);
@@ -364,7 +390,7 @@ Matrix& Matrix::parse (istream &is, Calculator &calc) {
 				break;
 			}
 			case 'K': {
-				char c = is.get();
+				int c = is.get();
 				if (c != 'X' && c != 'Y')
 					throw ParserException("transformation command 'K' must be followed by 'X' or 'Y'");
 				double a = getArgument(is, calc, 0, false, false);
@@ -374,9 +400,9 @@ Matrix& Matrix::parse (istream &is, Calculator &calc) {
 					throw ParserException(oss.str());
 				}
 				if (c == 'X')
-					xskew(a);
+					xskewByAngle(a);
 				else
-					yskew(a);
+					yskewByAngle(a);
             break;
 			}
 			case 'M': {
