@@ -2,7 +2,7 @@
 ** PageSize.h                                                           **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2013 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,8 +18,8 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#ifndef PAGESIZE_H
-#define PAGESIZE_H
+#ifndef DVISVGM_PAGESIZE_H
+#define DVISVGM_PAGESIZE_H
 
 #include "MessageException.h"
 
@@ -35,14 +35,14 @@ class PageSize
 		PageSize (std::string name);
 		void resize (std:: string name);
 		void resize (double w, double h);
-		double widthInPT () const  {return _width;}
-		double heightInPT () const {return _height;}
-		double widthInMM () const  {return _width*25.4/72.27;}
-		double heightInMM () const {return _height*25.4/72.27;}
+		double widthInBP () const  {return _width;}
+		double heightInBP () const {return _height;}
+		double widthInMM () const  {return _width*25.4/72;}
+		double heightInMM () const {return _height*25.4/72;}
 		bool valid () const        {return _width > 0 && _height > 0;}
 
    private:
-		double _width, _height; // in pt
+		double _width, _height; // in PS points
 };
 
 #endif
