@@ -2,7 +2,7 @@
 ** Process.h                                                            **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2013 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,8 +18,8 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#ifndef PROCESS_H
-#define PROCESS_H
+#ifndef DVISVGM_PROCESS_H
+#define DVISVGM_PROCESS_H
 
 #include <string>
 
@@ -27,7 +27,7 @@ class Process
 {
    public:
       Process (const std::string &cmd, const std::string &paramstr);
-      bool run (bool quiet=true);
+      bool run (std::string *out=0);
 
    protected:
       Process (const Process& orig) {}
