@@ -2,7 +2,7 @@
 ** PsSpecialHandler.cpp                                                 **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2013 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,6 +18,7 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
+#include <config.h>
 #include <cmath>
 #include <fstream>
 #include <iostream>
@@ -27,6 +28,7 @@
 #include "Ghostscript.h"
 #include "Message.h"
 #include "PSPattern.h"
+#include "PSPreviewFilter.h"
 #include "PsSpecialHandler.h"
 #include "SpecialActions.h"
 #include "XMLNode.h"
@@ -34,7 +36,6 @@
 
 using namespace std;
 
-
 static inline double str2double (const string &str) {
 	double ret;
 	istringstream iss(str);
@@ -43,13 +44,13 @@ static inline double str2double (const string &str) {
 }
 
 
-PsSpecialHandler::PsSpecialHandler () : _psi(this), _actions(0), _initialized(false), _xmlnode(0)
+PsSpecialHandler::PsSpecialHandler () : _psi(this), _actions(0), _previewFilter(_psi), _psSection(PS_NONE), _xmlnode(0)
 {
 }
 
 
 PsSpecialHandler::~PsSpecialHandler () {
-	_psi.setActions(0); // ensure no further PS actions are performed
+	_psi.setActions(0);     // ensure no further PS actions are performed
 	for (map<int, PSPattern*>::iterator it=_patterns.begin(); it != _patterns.end(); ++it)
 		delete it->second;
 }
@@ -58,8 +59,8 @@ PsSpecialHandler::~PsSpecialHandler () {
 /** Initializes the PostScript handler. It's called by the first use of process(). The
  *  deferred initialization speeds up the conversion of DVI files that doesn't contain
  *  PS specials. */
-void PsSpecialHandler::initialize (SpecialActions *actions) {
-	if (!_initialized) {
+void PsSpecialHandler::initialize () {
+	if (_psSection == PS_NONE) {
 		// initial values of graphics state
 		_linewidth = 1;
 		_linecap = _linejoin = 0;
@@ -71,26 +72,50 @@ void PsSpecialHandler::initialize (SpecialActions *actions) {
 
 		// execute dvips prologue/header files
 		const char *headers[] = {"tex.pro", "texps.pro", "special.pro", /*"color.pro",*/ 0};
-		for (const char **p=headers; *p; ++p) {
-			if (const char *path = FileFinder::lookup(*p, false)) {
-				ifstream ifs(path);
-				_psi.execute(ifs);
-			}
-			else
-				Message::wstream(true) << "PostScript header file " << *p << " not found\n";
+		for (const char **p=headers; *p; ++p)
+			processHeaderFile(*p);
+		_psSection = PS_HEADERS;  // allow to process header specials now
+	}
+}
+
+
+void PsSpecialHandler::processHeaderFile (const char *name) {
+	if (const char *path = FileFinder::lookup(name, false)) {
+		ifstream ifs(path);
+		_psi.execute(string("%%BeginProcSet: ")+name+" 0 0\n", false);
+		_psi.execute(ifs, false);
+		_psi.execute("%%EndProcSet\n", false);
+	}
+	else
+		Message::wstream(true) << "PostScript header file " << name << " not found\n";
+}
+
+
+void PsSpecialHandler::enterBodySection () {
+	if (_psSection == PS_HEADERS) {
+		_psSection = PS_BODY; // don't process any PS header code
+		ostringstream oss;
+		// process collected header code
+		if (!_headerCode.empty()) {
+			oss << "\nTeXDict begin @defspecial " << _headerCode << "\n@fedspecial end";
+			_headerCode.clear();
 		}
 		// push dictionary "TeXDict" with dvips definitions on dictionary stack
 		// and initialize basic dvips PostScript variables
-		ostringstream oss;
-		oss << " TeXDict begin 0 0 1000 72 72 () @start "
-		       " 0 0 moveto ";
-  		if (actions) {
+		oss << "\nTeXDict begin 0 0 1000 72 72 () @start 0 0 moveto ";
+  		if (_actions) {
 			float r, g, b;
-			actions->getColor().getRGB(r, g, b);
+			_actions->getColor().getRGB(r, g, b);
 			oss << r << ' ' << g << ' ' << b << " setrgbcolor ";
 		}
-		_psi.execute(oss.str());
-		_initialized = true;
+		_psi.execute(oss.str(), false);
+		// Check for information generated by preview.sty. If the tightpage options
+		// was set, don't execute the bop-hook but allow the PS filter to read
+		// the bbox data present at the beginning of the page.
+		_psi.setFilter(&_previewFilter);
+		_previewFilter.activate();
+		if (!_previewFilter.tightpage())
+			_psi.execute("userdict/bop-hook known{bop-hook}if\n", false);
 	}
 }
 
@@ -98,9 +123,8 @@ void PsSpecialHandler::initialize (SpecialActions *actions) {
 /** Move PS graphic position to current DVI location. */
 void PsSpecialHandler::moveToDVIPos () {
 	if (_actions) {
-		const double bp=72.0/72.27; // pt -> bp
-		const double x = _actions->getX()*bp;
-		const double y = _actions->getY()*bp;
+		const double x = _actions->getX();
+		const double y = _actions->getY();
 		ostringstream oss;
       oss << '\n' << x << ' ' << y << " moveto ";
       _psi.execute(oss.str());
@@ -118,16 +142,19 @@ void PsSpecialHandler::moveToDVIPos () {
 static void exec_and_syncpos (PSInterpreter &psi, istream &is, const DPair &pos, SpecialActions *actions) {
 	psi.execute(is);
 	psi.execute("\nquerypos ");   // retrieve current PS position (stored in 'pos')
-	const double pt = 72.27/72.0; // bp -> pt
-	actions->setX(pos.x()*pt);
-	actions->setY(pos.y()*pt);
+	if (actions) {
+		actions->setX(pos.x());
+		actions->setY(pos.y());
+	}
 }
 
 
+
 bool PsSpecialHandler::process (const char *prefix, istream &is, SpecialActions *actions) {
-	if (!_initialized)
-		initialize(actions);
 	_actions = actions;
+	initialize();
+	if (_psSection != PS_BODY && *prefix != '!' && strcmp(prefix, "header=") != 0)
+		enterBodySection();
 
 	if (*prefix == '"') {
 		// read and execute literal PostScript code (isolated by a wrapping save/restore pair)
@@ -137,21 +164,18 @@ bool PsSpecialHandler::process (const char *prefix, istream &is, SpecialActions
 		_psi.execute("\n@endspecial ");
 	}
 	else if (*prefix == '!') {
-		// execute literal PostScript header
-		_psi.execute("\nTeXDict begin @defspecial ");
-		_psi.execute(is);
-		_psi.execute("\n@fedspecial end ");
+		if (_psSection == PS_HEADERS) {
+			_headerCode += "\n";
+			_headerCode += string(istreambuf_iterator<char>(is), istreambuf_iterator<char>());
+		}
 	}
 	else if (strcmp(prefix, "header=") == 0) {
-		// read and execute PS header files
-		string fname;
-		is >> fname;
-		if (const char *path = FileFinder::lookup(fname, false)) {
-			ifstream ifs(path);
-			_psi.execute(ifs);
+		if (_psSection == PS_HEADERS) {
+			// read and execute PS header file
+			string fname;
+			is >> fname;
+			processHeaderFile(fname.c_str());
 		}
-		else
-			Message::estream(true) << "PS header file '" << fname << "' not found";
 	}
 	else if (strcmp(prefix, "psfile=") == 0 || strcmp(prefix, "PSfile=") == 0) {
 		if (_actions) {
@@ -163,7 +187,8 @@ bool PsSpecialHandler::process (const char *prefix, istream &is, SpecialActions
 		}
 	}
 	else if (strcmp(prefix, "ps::") == 0) {
-		_actions->finishLine();  // reset DVI position on next DVI command
+		if (_actions)
+			_actions->finishLine();  // reset DVI position on next DVI command
 		if (is.peek() == '[') {
 			// collect characters inside the brackets
 			string code;
@@ -189,7 +214,8 @@ bool PsSpecialHandler::process (const char *prefix, istream &is, SpecialActions
 		}
 	}
 	else { // ps: ...
-		_actions->finishLine();
+		if (_actions)
+			_actions->finishLine();
 		moveToDVIPos();
 		StreamInputReader in(is);
 		if (in.check(" plotfile ")) { // ps: plotfile fname
@@ -213,7 +239,7 @@ bool PsSpecialHandler::process (const char *prefix, istream &is, SpecialActions
 
 /** Handles psfile special.
  *  @param[in] fname EPS file to be included
- *  @param[in] attr attributes given with \special psfile */
+ *  @param[in] attr attributes given with \\special psfile */
 void PsSpecialHandler::psfile (const string &fname, const map<string,string> &attr) {
 	EPSFile epsfile(fname);
 	istream &is = epsfile.istream();
@@ -221,23 +247,22 @@ void PsSpecialHandler::psfile (const string &fname, const map<string,string> &at
 		Message::wstream(true) << "file '" << fname << "' not found in special 'psfile'\n";
 	else {
 		map<string,string>::const_iterator it;
-		const double pt = 72.27/72.0;  // bp -> pt
 
 		// bounding box of EPS figure
-		double llx = (it = attr.find("llx")) != attr.end() ? str2double(it->second)*pt : 0;
-		double lly = (it = attr.find("lly")) != attr.end() ? str2double(it->second)*pt : 0;
-		double urx = (it = attr.find("urx")) != attr.end() ? str2double(it->second)*pt : 0;
-		double ury = (it = attr.find("ury")) != attr.end() ? str2double(it->second)*pt : 0;
+		double llx = (it = attr.find("llx")) != attr.end() ? str2double(it->second) : 0;
+		double lly = (it = attr.find("lly")) != attr.end() ? str2double(it->second) : 0;
+		double urx = (it = attr.find("urx")) != attr.end() ? str2double(it->second) : 0;
+		double ury = (it = attr.find("ury")) != attr.end() ? str2double(it->second) : 0;
 
 		// desired width/height of resulting figure
-		double rwi = (it = attr.find("rwi")) != attr.end() ? str2double(it->second)/10.0*pt : -1;
-		double rhi = (it = attr.find("rhi")) != attr.end() ? str2double(it->second)/10.0*pt : -1;
+		double rwi = (it = attr.find("rwi")) != attr.end() ? str2double(it->second)/10.0 : -1;
+		double rhi = (it = attr.find("rhi")) != attr.end() ? str2double(it->second)/10.0 : -1;
 		if (rwi == 0 || rhi == 0 || urx-llx == 0 || ury-lly == 0)
 			return;
 
 		// user transformations (default values chosen according to dvips manual)
-		double hoffset = (it = attr.find("hoffset")) != attr.end() ? str2double(it->second)*pt : 0;
-		double voffset = (it = attr.find("voffset")) != attr.end() ? str2double(it->second)*pt : 0;
+		double hoffset = (it = attr.find("hoffset")) != attr.end() ? str2double(it->second) : 0;
+		double voffset = (it = attr.find("voffset")) != attr.end() ? str2double(it->second) : 0;
 //		double hsize   = (it = attr.find("hsize")) != attr.end() ? str2double(it->second) : 612;
 //		double vsize   = (it = attr.find("vsize")) != attr.end() ? str2double(it->second) : 792;
 		double hscale  = (it = attr.find("hscale")) != attr.end() ? str2double(it->second) : 100;
@@ -297,6 +322,63 @@ void PsSpecialHandler::psfile (const string &fname, const map<string,string> &at
 	}
 }
 
+
+/** Apply transformation to width, height, and depth set by preview package.
+ *  @param[in] matrix transformation matrix to apply
+ *  @param[out] w width
+ *  @param[out] h height
+ *  @param[out] d depth
+ *  @return true if the baseline is still horizontal after the transformation */
+static bool transform_box_extents (const Matrix &matrix, double &w, double &h, double &d) {
+	DPair shift = matrix*DPair(0,0);  // the translation component of the matrix
+	DPair ex = matrix*DPair(1,0)-shift;
+	DPair ey = matrix*DPair(0,1)-shift;
+	if (ex.y() != 0 && ey.x() != 0)  // rotation != mod 90 degrees?
+		return false;                 // => non-horizontal baseline, can't compute meaningful extents
+
+	if (ex.y() == 0)  // horizontal scaling or skewing?
+		w *= fabs(ex.x());
+	if (ey.x()==0 || ex.y()==0) { // vertical scaling?
+		if (ey.y() < 0) swap(h, d);
+		if (double sy = fabs(ey.y())/ey.length()) {
+			h *= fabs(ey.y()/sy);
+			d *= fabs(ey.y()/sy);
+		}
+		else
+			h = d = 0;
+	}
+	return true;
+}
+
+
+void PsSpecialHandler::dviEndPage (unsigned) {
+	BoundingBox bbox;
+	if (_previewFilter.getBoundingBox(bbox)) {
+		double w = _previewFilter.width();
+		double h = _previewFilter.height();
+		double d = _previewFilter.depth();
+		bool horiz_baseline = true;
+		if (_actions) {
+			_actions->bbox() = bbox;
+			// apply page transformations to box extents
+			Matrix pagetrans;
+			_actions->getPageTransform(pagetrans);
+			horiz_baseline = transform_box_extents(pagetrans, w, h, d);
+			_actions->bbox().lock();
+		}
+		Message::mstream() << "\napplying bounding box set by preview package (version " << _previewFilter.version() << ")\n";
+		if (horiz_baseline)
+			Message::mstream() << "width=" << XMLString(w) << "pt, " "height=" << XMLString(h) << "pt, " "depth=" << XMLString(d) << "pt\n";
+		else
+			Message::mstream() << "can't determine height, width, and depth due to non-horizontal baseline\n";
+	}
+	// close dictionary TeXDict and execute end-hook if defined
+	if (_psSection == PS_BODY) {
+		_psi.execute("\nend userdict/end-hook known{end-hook}if ");
+		_psSection = PS_HEADERS;
+	}
+}
+
 ///////////////////////////////////////////////////////
 
 void PsSpecialHandler::gsave (vector<double> &p) {
@@ -355,11 +437,6 @@ void PsSpecialHandler::stroke (vector<double> &p) {
 				bbox.transform(_actions->getMatrix());
 		}
 
-		const double pt = 72.27/72.0;  // factor to convert bp -> pt
-		ScalingMatrix scale(pt, pt);
-		_path.transform(scale);
-		bbox.transform(scale);
-
 		XMLElementNode *path=0;
 		Pair<double> point;
 		if (_path.isDot(point)) {  // zero-length path?
@@ -440,10 +517,6 @@ void PsSpecialHandler::fill (vector<double> &p, bool evenodd) {
 			if (!_xmlnode)
 				bbox.transform(_actions->getMatrix());
 		}
-		const double pt = 72.27/72.0;  // factor to convert bp -> pt
-		ScalingMatrix scale(pt, pt);
-		_path.transform(scale);
-		bbox.transform(scale);
 
 		ostringstream oss;
 		_path.writeSVG(oss);
@@ -586,11 +659,6 @@ void PsSpecialHandler::clip (vector<double> &p, bool evenodd) {
 		if (!_actions->getMatrix().isIdentity())
 			_path.transform(_actions->getMatrix());
 
-		const double pt = 72.27/72.0;  // factor to convert bp -> pt
-		ScalingMatrix scale(pt, pt);
-		_path.transform(scale);
-
-
 		int oldID = _clipStack.topID();
 		_clipStack.replace(_path);
 		int newID = _clipStack.topID();
@@ -765,7 +833,7 @@ void PsSpecialHandler::ClippingStack::pop (int saveID, bool grestoreall) {
 
 
 /** Returns a pointer to the path on top of the stack, or 0 if the stack is empty. */
-PsSpecialHandler::Path* PsSpecialHandler::ClippingStack::top () {
+const PsSpecialHandler::Path* PsSpecialHandler::ClippingStack::top () const {
 	return (!_stack.empty() && _stack.top().pathID)
 		? &_paths[_stack.top().pathID-1]
 		: 0;
