@@ -2,7 +2,7 @@
 ** StreamReader.h                                                       **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2013 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,8 +18,8 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#ifndef STREAMREADER_H
-#define STREAMREADER_H
+#ifndef DVISVGM_STREAMREADER_H
+#define DVISVGM_STREAMREADER_H
 
 #include <istream>
 #include <string>
@@ -47,6 +47,7 @@ class StreamReader
 		std::vector<UInt8>& readBytes (int n, std::vector<UInt8> &bytes, CRC32 &crc32);
 		int readByte () {return _is->get();}
 		int readByte (CRC32 &crc32);
+		void seek (std::istream::streampos pos, std::ios::seekdir dir) {_is->seekg(pos, dir);}
 
 	protected:
 		std::istream& in () {return *_is;}
