@@ -2,7 +2,7 @@
 ** Terminal.cpp                                                         **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2013 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,11 +18,8 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#include "Terminal.h"
-
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
+#include "Terminal.h"
 
 #ifdef HAVE_TERMIOS_H
 #include <termios.h>
@@ -55,6 +52,7 @@ const int Terminal::BLACK   = 0;
 
 #ifdef __WIN32__
 int Terminal::_defaultColor;
+int Terminal::_cursorHeight;
 #endif
 
 int Terminal::_fgcolor = Terminal::DEFAULT;
@@ -67,9 +65,12 @@ void Terminal::init (ostream &os) {
 #ifdef __WIN32__
 	HANDLE h = GetStdHandle(STD_OUTPUT_HANDLE);
 	if (h != INVALID_HANDLE_VALUE) {
-		CONSOLE_SCREEN_BUFFER_INFO info;
-		GetConsoleScreenBufferInfo(h, &info);
-		_defaultColor = (info.wAttributes & 0xff);
+		CONSOLE_SCREEN_BUFFER_INFO buffer_info;
+		GetConsoleScreenBufferInfo(h, &buffer_info);
+		_defaultColor = (buffer_info.wAttributes & 0xff);
+		CONSOLE_CURSOR_INFO cursor_info;
+		GetConsoleCursorInfo(h, &cursor_info);
+		_cursorHeight = cursor_info.dwSize;
 	}
 #endif
 }
@@ -80,6 +81,7 @@ void Terminal::init (ostream &os) {
 void Terminal::finish (ostream &os) {
 	fgcolor(DEFAULT, os);
 	bgcolor(DEFAULT, os);
+	cursor(true);
 }
 
 
@@ -189,3 +191,18 @@ void Terminal::bgcolor (int color, ostream &os) {
 		os << "\x1B[" << (40+(color & 0x07)) << 'm';
 #endif
 }
+
+
+/** Disables or enables the console cursor
+ *  @param[in] visible if false, the cursor is disabled, and enabled otherwise */
+void Terminal::cursor (bool visible) {
+#ifdef __WIN32__
+	HANDLE h = GetStdHandle(STD_OUTPUT_HANDLE);
+	if (h != INVALID_HANDLE_VALUE) {
+		CONSOLE_CURSOR_INFO cursor_info;
+		cursor_info.bVisible = visible;
+		cursor_info.dwSize = _cursorHeight;
+		SetConsoleCursorInfo(h, &cursor_info);
+	}
+#endif
+}
