@@ -2,7 +2,7 @@
 ** VFReader.cpp                                                         **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2013 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,6 +18,7 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
+#include <config.h>
 #include <sstream>
 #include "Font.h"
 #include "VFActions.h"
@@ -27,9 +28,10 @@
 using namespace std;
 
 
-/** Converts a TFM fix point value to double. */
+/** Converts a TFM fix point value to double (PS point units). */
 inline static double fix2double (FixWord fix) {
-	return double(fix)/(1 << 20);
+	const double pt2bp = 72/72.27;
+	return double(fix)/(1 << 20)*pt2bp;
 }
 
 
