@@ -2,7 +2,7 @@
 ** VFReader.h                                                           **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2013 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,8 +18,8 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#ifndef VFREADER_H
-#define VFREADER_H
+#ifndef DVISVGM_VFREADER_H
+#define DVISVGM_VFREADER_H
 
 #include "MessageException.h"
 #include "StreamReader.h"
