@@ -2,7 +2,7 @@
 ** XMLDocument.cpp                                                      **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2013 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,6 +18,7 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
+#include <config.h>
 #include "macros.h"
 #include "XMLDocument.h"
 
@@ -64,7 +65,7 @@ void XMLDocument::setRootNode (XMLElementNode *root) {
 
 ostream& XMLDocument::write (ostream &os) const {
 	if (_rootElement) { // no root element => no output
-		os << "<?xml version='1.0' encoding='ISO-8859-1'?>\n";
+		os << "<?xml version='1.0'?>\n";
 		FORALL(_nodes, list<XMLNode*>::const_iterator, i)
 			(*i)->write(os);
 		_rootElement->write(os);
@@ -77,7 +78,7 @@ ostream& XMLDocument::write (ostream &os) const {
  *  (this node won't be printed at all). If a node was only partly emitted, i.e.
  *  its child was the stop node, a further call of emit will continue the output.
  *  @param[in] os stream to which the output is written
- *  @param[in] stopElement node where emitting stops (if 0 the whole tree will be emitted)
+ *  @param[in] stopNode node where emitting stops (if 0 the whole tree will be emitted)
  *  @return true if node was completely emitted */
 bool XMLDocument::emit (ostream& os, XMLNode *stopNode) {
 	if (_rootElement) {  // no root element => no output
