@@ -26,10 +26,6 @@
 #include <efont/otfdata.hh>	// for ntohl()
 #include <efont/otfname.hh>
 
-#ifndef static_assert
-#define static_assert(c) switch (c) case 0: case (c):
-#endif
-
 namespace Efont { namespace OpenType {
 
 Vector<PermString> debug_glyph_names;
@@ -52,7 +48,7 @@ Font::parse_header(ErrorHandler *errh)
     int len = length();
     const uint8_t *data = this->data();
     if (HEADER_SIZE > len)
-	return errh->error("OTF too small for header"), -EFAULT;
+	return errh->error("OTF file corrupted (too small)"), -EFAULT;
     if ((data[0] != 'O' || data[1] != 'T' || data[2] != 'T' || data[3] != 'O')
 	&& (data[0] != '\000' || data[1] != '\001'))
 	return errh->error("not an OpenType font (bad magic number)"), -ERANGE;
@@ -76,7 +72,7 @@ Font::parse_header(ErrorHandler *errh)
 	if (tag <= last_tag)
 	    return errh->error("tags out of order"), -EINVAL;
 	if (offset + length > (uint32_t) len)
-	    return errh->error("OTF data for '%s' out of range", Tag(tag).text().c_str()), -EFAULT;
+	    return errh->error("OTF data for %<%s%> out of range", Tag(tag).text().c_str()), -EFAULT;
 	last_tag = tag;
     }
 
@@ -486,7 +482,7 @@ ScriptList::features(Tag script, Tag langsys, int &required_fid, Vector<int> &fi
     if (_str.length() < offset + LANGSYS_HEADERSIZE
 	|| (featureCount = Data::u16_aligned(data + offset + 4),
 	    (_str.length() < offset + LANGSYS_HEADERSIZE + featureCount*FEATURE_RECSIZE)))
-	return (errh ? errh->error("OTF LangSys table for '%s/%s' out of range", script.text().c_str(), langsys.text().c_str()) : -1);
+	return (errh ? errh->error("OTF LangSys table for %<%s/%s%> out of range", script.text().c_str(), langsys.text().c_str()) : -1);
 
     // search langsys table
     int f = Data::u16_aligned(data + offset + 2);
@@ -553,17 +549,17 @@ FeatureList::params(int fid, int length, ErrorHandler *errh, bool old_style_offs
     int len = _str.length();
     int nfeatures = Data::u16_aligned(data);
     if (fid < 0 || fid >= nfeatures)
-	return errh->error("OTF feature ID '%d' out of range", fid), String();
+	return errh->error("OTF feature ID %<%d%> out of range", fid), String();
     int foff = Data::u16_aligned(data + FEATURELIST_HEADERSIZE + fid*FEATURE_RECSIZE + 4);
     if (len < foff + FEATURE_HEADERSIZE)
-	return errh->error("OTF LookupList for feature ID '%d' too short", fid), String();
+	return errh->error("OTF LookupList for feature ID %<%d%> too short", fid), String();
     int poff = Data::u16_aligned(data + foff);
     if (poff == 0)
 	return String();
     if (!old_style_offset)
 	poff += foff;
     if (len < poff + length)
-	return errh->error("OTF feature parameters for feature ID '%d' out of range", fid), String();
+	return errh->error("OTF feature parameters for feature ID %<%d%> out of range", fid), String();
     else
 	return _str.substring(poff, length);
 }
@@ -673,13 +669,13 @@ FeatureList::lookups(int fid, Vector<int> &results, ErrorHandler *errh, bool cle
     int len = _str.length();
     int nfeatures = Data::u16_aligned(data);
     if (fid < 0 || fid >= nfeatures)
-	return errh->error("OTF feature ID '%d' out of range", fid);
+	return errh->error("OTF feature ID %<%d%> out of range", fid);
     int foff = Data::u16_aligned(data + FEATURELIST_HEADERSIZE + fid*FEATURE_RECSIZE + 4);
     int lookupCount;
     if (len < foff + FEATURE_HEADERSIZE
 	|| (lookupCount = Data::u16_aligned(data + foff + 2),
 	    len < foff + FEATURE_HEADERSIZE + lookupCount*LOOKUPLIST_RECSIZE))
-	return errh->error("OTF LookupList for feature ID '%d' too short", fid);
+	return errh->error("OTF LookupList for feature ID %<%d%> too short", fid);
     const uint8_t *ldata = data + foff + FEATURE_HEADERSIZE;
     for (int j = 0; j < lookupCount; j++, ldata += LOOKUPLIST_RECSIZE)
 	results.push_back(Data::u16_aligned(ldata));
@@ -815,7 +811,7 @@ Coverage::check(ErrorHandler *errh)
     // USHORT	glyphCount
     const uint8_t *data = _str.udata();
     if (_str.length() < HEADERSIZE)
-	return errh->error("OTF coverage table too small for header");
+	return errh->error("OTF coverage table too small");
     int coverageFormat = Data::u16_aligned(data);
     int count = Data::u16_aligned(data + 2);
 
@@ -1252,7 +1248,7 @@ ClassDef::check(ErrorHandler *errh)
     // USHORT	glyphCount
     const uint8_t *data = _str.udata();
     if (_str.length() < 6)	// NB: prevents empty format-2 tables
-	return errh->error("OTF class def table too small for header");
+	return errh->error("OTF class def table too small");
     int classFormat = Data::u16_aligned(data);
 
     int len;
