@@ -1,6 +1,6 @@
 /* luagen.h
    
-   Copyright 2009 Taco Hoekwater <taco@luatex.org>
+   Copyright 2009-2013 Taco Hoekwater <taco@luatex.org>
 
    This file is part of LuaTeX.
 
@@ -17,11 +17,11 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
-/* $Id: luagen.h 3261 2009-12-18 11:38:21Z taco $ */
+/* $Id: luagen.h 4679 2013-12-19 15:47:53Z luigi $ */
 
 extern void lua_begin_page(PDF pdf);
 extern void lua_end_page(PDF pdf);
 
-extern void lua_place_glyph(PDF pdf, internal_font_number f, int c);
+extern void lua_place_glyph(PDF pdf, internal_font_number f, int c, int ex);
 extern void lua_place_rule(PDF pdf, scaledpos size);
 extern void finish_lua_file(PDF pdf);
