@@ -27,8 +27,8 @@
 
 /* utils.c */
 __attribute__ ((format(printf, 1, 2)))
-extern void pdftex_warn(const char *fmt, ...);
+extern void luatex_warn(const char *fmt, ...);
 __attribute__ ((noreturn, format(printf, 1, 2)))
-extern void pdftex_fail(const char *fmt, ...);
+extern void luatex_fail(const char *fmt, ...);
 
 #endif /* LUATEX_COMMON_H */
