@@ -1,5 +1,5 @@
 /*
-Copyright 1996-2012 Han The Thanh, <thanh@pdftex.org>
+Copyright 1996-2014 Han The Thanh, <thanh@pdftex.org>
 
 This file is part of pdfTeX.
 
@@ -521,15 +521,15 @@ static void read_font(void)
             }
         }
         break;
+    default:
+        ttf_warn("unsupported format (%.8X) of `post' table, assuming 3.0",
+                 post_format);
     case 0x00030000:
         if (print_glyph == AS_NAME) {
             ttf_warn
                 ("no names available in `post' table, print glyph names as indices");
             print_glyph = AS_INDEX;
         }
-        break;
-    default:
-        ttf_fail("unsupported format (%.8X) of `post' table", post_format);
     }
     ttf_seek_tab("loca", 0);
     if (loca_format == 1) {
@@ -1050,7 +1050,7 @@ int main(int argc, char **argv)
         case 'v':
             fprintf(stderr,
                     "ttf2afm version " VERSION "\n"
-                    "Copyright (C) 1997-2011 Han The Thanh.\n"
+                    "Copyright (C) 1997-2014 Han The Thanh.\n"
                     "There is NO warranty.  Redistribution of this software is\n"
                     "covered by the terms of both the pdfTeX copyright and\n"
                     "the GNU General Public License.\n"
