@@ -1,5 +1,5 @@
 /*
-Copyright 1996-2012 Han The Thanh, <thanh@pdftex.org>
+Copyright 1996-2014 Han The Thanh, <thanh@pdftex.org>
 
 This file is part of pdfTeX.
 
@@ -517,14 +517,13 @@ static void ttf_read_post(void)
             }
         }
         break;
+    default:
+        pdftex_warn("unsupported format (%.8X) of `post' table, assuming 3.0",
+                    (unsigned int) post_format);
     case 0x00030000:
         for (glyph = glyph_tab; glyph - glyph_tab < NMACGLYPHS; glyph++) {
             glyph->name_index = glyph - glyph_tab;
         }
-        break;
-    default:
-        pdftex_fail("unsupported format (%.8X) of `post' table",
-                    (unsigned int) post_format);
     }
 }
 
